#include <3ds.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "filesystem.h"

Result write_savedata(char* path, u8* data, u32 size)
{
	if(!path || !data || !size)return -1;

	Handle outFileHandle;
	u32 bytesWritten=0;
	Result ret = 0;
	int fail = 0;

	disableHBLHandle();

	ret = FSUSER_OpenFile(&outFileHandle, saveGameArchive, fsMakePath(PATH_ASCII, path), FS_OPEN_CREATE | FS_OPEN_WRITE, 0);
	if(ret){fail = -8; goto writeFail;}

	ret = FSFILE_Write(outFileHandle, &bytesWritten, 0x0, data, size, 0x10001);
	if(ret){fail = -9; goto writeFail;}

	ret = FSFILE_Close(outFileHandle);
	if(ret){fail = -10; goto writeFail;}

	ret = FSUSER_ControlArchive(saveGameArchive, ARCHIVE_ACTION_COMMIT_SAVE_DATA, NULL, 0, NULL, 0);

	writeFail:
	if(fail)printf("write fail: %d %08X %08X\n", fail, (unsigned int)ret, (unsigned int)bytesWritten);
	else printf("successfully wrote to file ! %08X\n", (unsigned int)bytesWritten);

	enableHBLHandle();

    return ret;
}

Result install2DSaver(char *input, int slot){
	Result res=0;
	u32 tx=0;
	char filename[100]={0};
	char savename[100]={0};
	u8 *wbuf=malloc(0x2000);
	res = filesystemInit();
	if(res) return res;
	
	printf("Opening hax save ...\n");
	sprintf(filename, "romfs:/%s", input);
	FILE *sv=fopen(filename,"rb");
	if(!sv) return 1;
	tx = fread(wbuf, 1, 0x14DC, sv);
	if(!tx) return 2;
	fclose(sv);
	sprintf(savename,"/save0%d.bin", slot);
	res = write_savedata(savename, wbuf, 0x14DC);
	if(res) return res;
	
	printf("Hax save written, opening payload ...\n");
	
	FILE *pl=fopen("romfs:/2DSaver.bin","rb");
	if(!pl) return 3;
	tx = 0;
	tx = fread(wbuf, 1, 0xF58, pl);
	if(!tx) return 4;
	fclose(pl);
	res = write_savedata("/2DSaver.bin", wbuf, 0xF58);
	free(wbuf);
	if(res) return res;
	
	printf("\nInstall success!\n");
	return res;
}

void checkSlots(){
	Result res=0;
	Handle outFileHandle;
	u32 bytesRead=0;
	res = filesystemInit();
	disableHBLHandle();
	if(res) printf("filesystem cannot init!\n");
	char slotname[100];
	char s[17];
	printf("[Slot-1]    [Slot-2]    [Slot-3]\n");
	for(int i=0; i<3; i++){
		memset(slotname, 0, 100);
		memset(s, 0, 17);
		sprintf(slotname, "/save0%d.bin", i);
		res = FSUSER_OpenFile(&outFileHandle, saveGameArchive, fsMakePath(PATH_ASCII, slotname), FS_OPEN_READ, 0);
		FSFILE_Read(outFileHandle, &bytesRead, 0x1C, s, 17);
		if(!res){
			bytesRead = (u32)*(s+16) * 2; //oot's name length field
			if(bytesRead > 16) bytesRead = 16; //ignore yellows8's length suggestion :p
			for(int j=0; j < 16; j+=2){
				if(j < bytesRead) printf("%c",*(s+j));
				else      printf(" ");
			}
			printf("    ");
		}
		else printf("Open!!     ");
		FSFILE_Close(outFileHandle);
	}
	enableHBLHandle();
	printf("\n\n");
}

int main(int argc, char **argv)
{
	gfxInitDefault();
	consoleInit(GFX_TOP, NULL);
	romfsInit();
	Result res=0;
	int once=0;
	int slot=0;
	u64 pid=0;
	char filename[20]={0};


	printf("2DSaver_installer - zoogie\n\n");
	
	res = APT_GetProgramID(&pid);
	
	switch(pid){
		case 0x0004000000033400: printf("JPN OOT gamecard detected!\n"); strcpy(filename,"jpn"); break;
		case 0x0004000000033500: printf("USA OOT gamecard detected!\n"); strcpy(filename,"usa"); break;
		case 0x0004000000033600: printf("EUR OOT gamecard detected!\n"); strcpy(filename,"eur"); break;
		case 0x000400000008f800: printf("KOR OOT gamecard detected!\n"); strcpy(filename,"kor"); break;
		case 0x000400000008f900: printf("CHN/TWN OOT gamecard detected!\n"); strcpy(filename,"chntwn"); break;
		default: printf("No OOT gamecard detected!\n");
	}
	
	
	printf("\n");
	printf("--[\x1b[33;1mPress A to install 2DSaver\x1b[37;0m]--\n");
	printf("--[\x1b[33;1mPress Y to select saveslot\x1b[37;0m]--\n");
	printf("--[\x1b[33;1mPress Start to exit       \x1b[37;0m]--\n\n");
	
	checkSlots();
	printf("\rSlot-1 selected");

	while (aptMainLoop())
	{
		hidScanInput();

		u32 kDown = hidKeysDown();

		if (kDown & KEY_Y){
			slot++;
			if(slot>2)slot=0;
			if(once==0) printf("\rSlot-%d selected", slot + 1);
		}
		else if (kDown & KEY_A){
			if(once==0) {
				printf("\n\n");
				if((res = install2DSaver(filename, slot))) printf("Installation failure\n");
				printf("%08X\n",(int)res);
				once=1;
			}
		}
		else if (kDown & KEY_START) break; 

		gfxFlushBuffers();
		gfxSwapBuffers();

		gspWaitForVBlank();
	}

	gfxExit();
	return 0;
}